@@ -15,6 +15,8 @@
 /*   To compile and use:                                                  */
 /*     See INSTALL file.                                                  */
 
+#include "internal.h"
+
 #include <string.h>
 #include <assert.h>
 
@@ -416,7 +418,7 @@ void PE_populate_key_list_view(key_ring *ring, gboolean local, GtkTreeView* key_
                     key_iter->name, key_iter->account, num);
       } else {
          key_buf = g_string_new("--error--");
-         purple_debug(PURPLE_DEBUG_WARNING, "pidgin-encryption", "Error Setting List Item %p %p %p\n",
+         purple_debug(PURPLE_DEBUG_WARNING, "pidgin-encryption", "Error Setting List Item %p %p\n",
                     key_iter->key , key_iter->key->fingerprint);
       }
 
