@@ -17,6 +17,8 @@
 
 #define PURPLE_PLUGINS
 
+#include "internal.h"
+
 #include "pidgin-encryption-config.h"
 
 #include <gdk/gdk.h>
@@ -126,12 +128,20 @@ static void PE_new_conv_cb(PurpleConversation *, void *);
 static void PE_del_conv_cb(PurpleConversation *, void *);
 static void PE_updated_conv_cb(PurpleConversation *, void *);
 
+/* legacy... we try to use HTML in some of our headers- format is protocol dependent */
 static GHashTable *header_table, *footer_table, *notify_table;
-static gchar* header_default;
+static gchar* header_default;  /* the non-HTML default header */
+
+static gchar* header_broken;   /* if a server is stripping HTML and we're using it */
+                               /* in our header, we'll see this                    */
+static GHashTable *broken_users;  /* keeps track of who is seeing broken HTML */
 
 static char * unrequited_capable_who = 0;  /* if we learn that someone is capable, but don't have */
                                            /* a conv for them yet, we set this to be them         */
 
+/* A field for the LibPurple conversation "data" hashmap to indicate that we want to use non-html */
+#define BROKEN_HTML "Encrypt-HTMLBroken"
+
 static void strip_crypto_smiley(char* s) {
    char * pos;
    
@@ -149,22 +159,27 @@ static void PE_send_key(PurpleAccount *acct, const char *name, int asError, gcha
    GString *key_str;
    crypt_key *pub_key;
    PurpleConversation *conv;
+   int conv_breaks_html = 0;
 
    int header_size, footer_size;
    const gchar* header = g_hash_table_lookup(header_table, purple_account_get_protocol_id(acct));
    const gchar* footer = g_hash_table_lookup(footer_table, purple_account_get_protocol_id(acct));
 
-   if (!header) header = header_default;
-   if (!footer) footer = "";
-
-   header_size = strlen(header);
-   footer_size = strlen(footer);
-
    purple_debug(PURPLE_DEBUG_INFO, "pidgin-encryption", "send_key: %s\n", acct->username);
    
    conv = purple_find_conversation_with_account(PURPLE_CONV_TYPE_IM, name, acct);
    purple_debug(PURPLE_DEBUG_INFO, "pidgin-encryption", "send_key: %s, %p, %s\n", name, conv, acct->username);
 
+   if (g_hash_table_lookup(broken_users, name)) {
+      conv_breaks_html = 1;
+   }
+
+   if (!header || conv_breaks_html) header = header_default;
+   if (!footer || conv_breaks_html) footer = "";
+
+   header_size = strlen(header);
+   footer_size = strlen(footer);
+
    pub_key = PE_find_own_key_by_name(&PE_my_pub_ring, acct->username, acct, conv);
    purple_debug(PURPLE_DEBUG_INFO, "pidgin-encryption", "send_key2: %s\n", acct->username);
    if (!pub_key) return;
@@ -188,8 +203,8 @@ static void PE_send_key(PurpleAccount *acct, const char *name, int asError, gcha
    }
 
    if (strlen(msg) > PE_get_msg_size_limit(acct)) {
-      purple_debug(PURPLE_DEBUG_MISC, "pidgin-encryption", "Key too big to send in message (%d > %d)\n", 
-                 strlen(msg), PE_get_msg_size_limit(acct));
+      purple_debug(PURPLE_DEBUG_MISC, "pidgin-encryption", "Key too big to send in message (%u > %d)\n", 
+                   (unsigned)strlen(msg), PE_get_msg_size_limit(acct));
       conv = purple_find_conversation_with_account(PURPLE_CONV_TYPE_IM, name, acct);
       if (conv == NULL) {
          conv = purple_conversation_new(PURPLE_CONV_TYPE_IM, acct, name);
@@ -205,27 +220,35 @@ static void PE_send_key(PurpleAccount *acct, const char *name, int asError, gcha
    g_string_free(key_str, TRUE);
 }
 
-
-
 static crypt_key *PE_get_key(PurpleConnection *gc, const char *name) {
    crypt_key *bkey;
-   char* tmpmsg;
-
-   int header_size, footer_size;
-   const gchar* header = g_hash_table_lookup(header_table, purple_account_get_protocol_id(gc->account));
-   const gchar* footer = g_hash_table_lookup(footer_table, purple_account_get_protocol_id(gc->account));
-
-   if (!header) header = header_default;
-   if (!footer) footer = "";
-
-   header_size = strlen(header);
-   footer_size = strlen(footer);
 
    purple_debug(PURPLE_DEBUG_INFO, "pidgin-encryption", "get_key: %s\n", name);
    bkey = PE_find_key_by_name(PE_buddy_ring, name, gc->account);
    purple_debug(PURPLE_DEBUG_INFO, "pidgin-encryption", "got key: %p\n", bkey);
 
    if( bkey == 0 ) {
+      char* tmpmsg;
+      
+      int header_size, footer_size;
+      const gchar* header = g_hash_table_lookup(header_table, purple_account_get_protocol_id(gc->account));
+      const gchar* footer = g_hash_table_lookup(footer_table, purple_account_get_protocol_id(gc->account));
+      int conv_breaks_html = 0;
+
+      if (g_hash_table_lookup(broken_users, name)) {
+         conv_breaks_html = 1;
+      }
+      if (g_hash_table_lookup(broken_users, name)) {
+         conv_breaks_html = 1;
+      }
+   
+      
+      if (!header || conv_breaks_html) header = header_default;
+      if (!footer || conv_breaks_html) footer = "";
+      
+      header_size = strlen(header);
+      footer_size = strlen(footer);
+
       tmpmsg = alloca(header_size + footer_size +
                       sizeof (": Send Key")); // sizeof() gets the trailing null too
 
@@ -317,10 +340,10 @@ static void PE_store_msg(const char *who, PurpleConnection *gc, char *msg, msg_n
    }
 
    for (newnode = *first_node; newnode != *last_node; newnode = newnode->next) {
-      purple_debug(PURPLE_DEBUG_INFO, "pidgin-encryption", "   In store stack: %p\n",
+      purple_debug(PURPLE_DEBUG_INFO, "pidgin-encryption", "   In store stack: %p, %s\n",
                  newnode, newnode->who);
    }
-   purple_debug(PURPLE_DEBUG_INFO, "pidgin-encryption", "   In store stack: %p\n",
+   purple_debug(PURPLE_DEBUG_INFO, "pidgin-encryption", "   In store stack: %p, %s\n",
               *last_node, (*last_node)->who);
 }
 
@@ -505,6 +528,8 @@ static gboolean PE_got_msg_cb(PurpleAccount *acct, char **who, char **message,
    gchar *caps_header, *caps_message,    /* temps for ascii_strup() versions of each */
          *caps_notify;                   /* since Jabber mucks with case             */
 
+   gchar *boldAsterixPos;                /* position of <b>*</b>-substituted header */
+   gchar *debianHeaderPos;               /* position of Debian-specific header */
    gchar *unescaped_message;             /* temps for html_unescaped       */
                                          /* since ICQ will now escape HTML */
 
@@ -525,9 +550,26 @@ static gboolean PE_got_msg_cb(PurpleAccount *acct, char **who, char **message,
    name = g_strdup(purple_normalize(acct, *who));
    
    if (*message != NULL) {
+      /* More header madness: Debian patched the headers to start with
+         "--- Encrypted with the ...", to fix the issue that sometimes
+         "***" is being replaced with <b>*</b>.  So... replace either that
+         we see, to canonicalize the message */
+
+      /* also make message all caps... */
       caps_message = g_ascii_strup(*message, -1);
       caps_header = g_ascii_strup(header, -1);
 
+      boldAsterixPos = strstr(caps_message, "<B>*</B> ENCRYPTED WITH THE GAIM-ENCRYPTION PLUGIN");
+      if (boldAsterixPos) {
+          memcpy(boldAsterixPos, "     ***", 8);
+      }
+
+      debianHeaderPos = strstr(caps_message, "--- ENCRYPTED WITH THE GAIM-ENCRYPTION PLUGIN");
+
+      if (debianHeaderPos) {
+          memcpy(debianHeaderPos, "***", 3);
+      }
+
       headerpos = strstr(caps_message, caps_header);
       g_free(caps_header);
 
@@ -561,7 +603,8 @@ static gboolean PE_got_msg_cb(PurpleAccount *acct, char **who, char **message,
          if (headerpos != 0 || notifypos != 0) {
             /* ICQ PRPL escaped our HTML header, but we outsmarted it     */
             /* replace message with unescaped message.                    */
-            purple_debug(PURPLE_DEBUG_MISC, "pidgin-encryption", "Escaped header: replacing %s with %s\n",
+            purple_debug(PURPLE_DEBUG_MISC, "pidgin-encryption",
+                         "Escaped header: replacing %s with %s\n",
                        *message, unescaped_message);
             g_free(*message);
             *message = unescaped_message;
@@ -570,8 +613,41 @@ static gboolean PE_got_msg_cb(PurpleAccount *acct, char **who, char **message,
          }
       }
 
-      /* Whew.  Enough of this header-finding.  */
+      if (headerpos == 0 && notifypos == 0) {
+         /* check for a header that has had the HTML ripped out. */
+         if (strstr(*message, header_broken)) {
+            /* mark this name as having broken HTML, so we send appropriately */
+            g_hash_table_insert(broken_users, g_strdup(name), (gpointer)TRUE);
+            /* send key to other side as an error condition */
+            PE_send_key(acct, name, 1, 0);
+            (*message)[0] = 0;
+            g_free(*message);
+            *message = NULL;
+            purple_debug(PURPLE_DEBUG_MISC, "pidgin-encryption",
+                         "Broken HTML header found, asking for key\n");
+            
+            // Bail early, since we short-circuited the logic below
+            g_free(name);
+            return FALSE;
+         }
+      }
 
+      if (headerpos == 0 && header != header_default){
+         /* look for a default header, in case other side has decided that html is broken */
+         headerpos = strstr(*message, header_default);
+         if (headerpos) {
+            purple_debug(PURPLE_DEBUG_MISC, "pidgin-encryption",
+                         "Found default header when expecting proto-specific one\n");
+            header_size = strlen(header_default);
+            footer_size = 0;
+
+            /* mark this conv as having broken HTML, so we send appropriately */
+            g_hash_table_insert(broken_users, g_strdup(name), (gpointer)TRUE);
+         }
+      }
+
+      /* Whew.  Enough of this header-finding.  */
+      
       if (headerpos != 0) {
          PE_set_capable(conv, TRUE);
          if (purple_prefs_get_bool("/plugins/gtk/encrypt/encrypt_response")) {
@@ -586,31 +662,31 @@ static gboolean PE_got_msg_cb(PurpleAccount *acct, char **who, char **message,
             purple_debug(PURPLE_DEBUG_MISC, "pidgin-encryption", "Sent key per request\n");
          } else if (strncmp(headerpos + header_size, ": Key",
                             sizeof(": Key") - 1) == 0) {
-            purple_debug(PURPLE_DEBUG_MISC, "pidgin-encryption", "Got key\n");
-            PE_received_key(headerpos + header_size + sizeof(": Key") - 1, name, acct,
-                            conv, message);
+               purple_debug(PURPLE_DEBUG_MISC, "pidgin-encryption", "Got key\n");
+               PE_received_key(headerpos + header_size + sizeof(": Key") - 1, name, acct,
+                               conv, message);
          } else if (strncmp(headerpos + header_size, ": ErrKey",
                             sizeof(": ErrKey") - 1) == 0) {
             purple_debug(PURPLE_DEBUG_MISC, "pidgin-encryption", "Got key in response to error\n");
             purple_conversation_write(conv, 0,
-                                    _("Last outgoing message not received properly- resetting"),
-                                    PURPLE_MESSAGE_SYSTEM, time((time_t)NULL));
-
+                                      _("Last outgoing message not received properly- resetting"),
+                                      PURPLE_MESSAGE_SYSTEM, time((time_t)NULL));
+            
             PE_received_key(headerpos + header_size + sizeof(": ErrKey") - 1, name, acct,
                             conv, message);
          } else if (strncmp(headerpos + header_size, ": Msg",
                             sizeof(": Msg") - 1) == 0){
             purple_debug(PURPLE_DEBUG_MISC, "pidgin-encryption",
-                       "Got encrypted message: %d\n", strlen(*message)); 
+                         "Got encrypted message: %u\n", (unsigned)strlen(*message)); 
             purple_debug(PURPLE_DEBUG_INFO, "pidgin-encryption",
-                       "Message is:%s:\n", *message);
+                         "Message is:%s:\n", *message);
             memmove(*message, headerpos + header_size + sizeof(": Msg") - 1,
                     strlen(headerpos + header_size + sizeof(": Msg") -1)+1);
             got_encrypted_msg(acct->gc, name, message);
             PE_set_rx_encryption(conv, TRUE);
          } else {
             purple_debug(PURPLE_DEBUG_ERROR, "pidgin-encryption",
-                       "Invalid Pidgin-Encryption packet type\n"); 
+                         "Invalid Pidgin-Encryption packet type\n"); 
          }
       } else if (notifypos != 0) {
          PE_set_rx_encryption(conv, FALSE);
@@ -635,7 +711,7 @@ static gboolean PE_got_msg_cb(PurpleAccount *acct, char **who, char **message,
          PE_set_rx_encryption(conv, FALSE);
          purple_debug(PURPLE_DEBUG_MISC, "pidgin-encryption", "No header: %s\n", *message);
          purple_debug(PURPLE_DEBUG_MISC, "pidgin-encryption",
-                    "Proto '%s', header should be: %s\n", purple_account_get_protocol_id(acct), header);
+                      "Proto '%s', header should be: %s\n", purple_account_get_protocol_id(acct), header);
          strip_crypto_smiley(*message);
       }
    }
@@ -773,10 +849,11 @@ static int PE_get_msg_size_limit(PurpleAccount *acct) {
 
 static void PE_send_msg_cb(PurpleAccount *acct, char *who, char **message, void* data) {
    char *out_msg, *crypt_msg = 0;
-   char *dupname;
+   char *dupname = g_strdup(purple_normalize(acct, who));
+
    int msgsize;
    const char msg_format[] = "%s: Msg:S%.10s:R%.10s: Len %d:%s%s";
-   PurpleConversation *conv;
+   PurpleConversation *conv = purple_find_conversation_with_account(PURPLE_CONV_TYPE_IM, who, acct);
    crypt_key *our_key, *his_key;
    GSList *cur_msg;
    GQueue *sent_msg_queue;
@@ -790,9 +867,15 @@ static void PE_send_msg_cb(PurpleAccount *acct, char *who, char **message, void*
    const gchar* footer = g_hash_table_lookup(footer_table, purple_account_get_protocol_id(acct));
    const gchar* notify = g_hash_table_lookup(notify_table, purple_account_get_protocol_id(acct));
 
-   if (!header) header = header_default;
-   if (!footer) footer = "";
-
+   int conv_breaks_html = 0;
+   
+   if (g_hash_table_lookup(broken_users, dupname)) {
+      conv_breaks_html = 1;
+   }
+   
+   if (!header || conv_breaks_html) header = header_default;
+   if (!footer || conv_breaks_html) footer = "";
+   
    msg_size_limit = PE_get_msg_size_limit(acct);
 
    /* who: name that you are sending to */
@@ -805,11 +888,10 @@ static void PE_send_msg_cb(PurpleAccount *acct, char *who, char **message, void*
 
    /* Message might have been eaten by another plugin: */
    if ((message == NULL) || (*message == NULL)) {
+      g_free(dupname);
       return;
    }
 
-   conv = purple_find_conversation_with_account(PURPLE_CONV_TYPE_IM, who, acct);
-
    if (conv == NULL) {
       conv = purple_conversation_new(PURPLE_CONV_TYPE_IM, acct, who);
    }
@@ -832,6 +914,7 @@ static void PE_send_msg_cb(PurpleAccount *acct, char *who, char **message, void*
          *message = out_msg;
       }
       purple_debug(PURPLE_DEBUG_MISC, "pidgin-encryption", "Outgoing Msg::%s::\n", *message);
+      g_free(dupname);
       return;
    }
 
@@ -845,10 +928,10 @@ static void PE_send_msg_cb(PurpleAccount *acct, char *who, char **message, void*
                       /* find_own_key (above) will have displayed error messages */
 		purple_debug(PURPLE_DEBUG_MISC, "pidgin-encryption", "leaving\n");
 
+      g_free(dupname);
       return;
    }
 
-   dupname = g_strdup(purple_normalize(acct, who));
    his_key = PE_get_key(acct->gc, dupname);
 
    if (his_key == 0) { /* Don't have key for this guy yet */
@@ -861,7 +944,7 @@ static void PE_send_msg_cb(PurpleAccount *acct, char *who, char **message, void*
       PE_store_msg(who, acct->gc, *message, &first_out_msg, &last_out_msg);
 
    } else {  /* We have a key.  Encrypt and send. */
-      purple_debug(PURPLE_DEBUG_INFO, "pidgin-encryption", "has key\n", dupname);
+      purple_debug(PURPLE_DEBUG_INFO, "pidgin-encryption", "has key (%s)\n", dupname);
       baggage_size = snprintf(baggage, sizeof(baggage), msg_format, header, our_key->digest,
                              his_key->digest, 10000, "", footer);
       baggage_size = MIN(baggage_size, sizeof(baggage) - 1);
@@ -906,7 +989,7 @@ static void PE_send_msg_cb(PurpleAccount *acct, char *who, char **message, void*
 
          g_queue_push_head(sent_msg_queue, sent_msg_item);
 
-         purple_debug(PURPLE_DEBUG_INFO, "pidgin-encryption", "Enc for send: '%s'\n", cur_msg->data);
+         purple_debug(PURPLE_DEBUG_INFO, "pidgin-encryption", "Enc for send: '%s'\n", (char*)cur_msg->data);
 
          PE_encrypt_signed(&crypt_msg, cur_msg->data, our_key, his_key);
          msgsize = strlen(crypt_msg);
@@ -924,7 +1007,7 @@ static void PE_send_msg_cb(PurpleAccount *acct, char *who, char **message, void*
 							 acct, purple_conversation_get_name(conv), out_msg);
 
          purple_debug(PURPLE_DEBUG_INFO, "pidgin-encryption",
-                    "send_im: %s: %d\n", who, strlen(out_msg));
+                      "send_im: %s: %u\n", who, (unsigned)strlen(out_msg));
          purple_debug(PURPLE_DEBUG_INFO, "pidgin-encryption",
                     "outgoing:%s:\n", out_msg);
          g_free(out_msg);
@@ -956,6 +1039,7 @@ void PE_resend_msg(PurpleAccount* acct, const char* name, gchar *msg_id) {
    int baggage_size;
    char baggage[PE_BUF_LONG];
    const gchar *header, *footer;
+   int conv_breaks_html = 0;
 
    if (msg_id == 0) {
       purple_debug(PURPLE_DEBUG_ERROR, "pidgin-encryption", "Bad call to resend_msg: %p %p\n", conv, msg_id);
@@ -969,9 +1053,13 @@ void PE_resend_msg(PurpleAccount* acct, const char* name, gchar *msg_id) {
    header = g_hash_table_lookup(header_table, purple_account_get_protocol_id(acct));
    footer = g_hash_table_lookup(footer_table, purple_account_get_protocol_id(acct));
 
-   if (!header) header = header_default;
-   if (!footer) footer = "";
-
+   if (g_hash_table_lookup(broken_users, name)) {
+      conv_breaks_html = 1;
+   }
+   
+   if (!header || conv_breaks_html) header = header_default;
+   if (!footer || conv_breaks_html) footer = "";
+   
    /*Sometimes callers don't know whether there's a msg to send... */
    if (msg_id == 0 || conv == 0) return;
 
@@ -1030,7 +1118,7 @@ void PE_resend_msg(PurpleAccount* acct, const char* name, gchar *msg_id) {
          serv_send_im(conv->account->gc, name, out_msg, 0);
          
          purple_debug(PURPLE_DEBUG_INFO, "pidgin-encryption",
-                    "resend_im: %s: %d\n", name, strlen(out_msg));
+                      "resend_im: %s: %u\n", name, (unsigned)strlen(out_msg));
          purple_debug(PURPLE_DEBUG_INFO, "pidgin-encryption",
                     "resend outgoing:%s:\n", out_msg);
          g_free(msg);
@@ -1151,6 +1239,10 @@ static void PE_headers_init() {
                        g_strdup("<A HREF='Gaim-Encryption Capable'> </A>"));
 
    header_default = g_strdup("*** Encrypted :");
+
+   header_broken = g_strdup("*** Encrypted with the Gaim-Encryption plugin");
+
+   broken_users = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
 }
 
 /* #define CRYPT_HEADER "*** Encrypted with the Gaim-Encryption plugin <A HREF=\"" */
