@@ -15,6 +15,8 @@
 /*   To compile and use:                                                  */
 /*     See INSTALL file.                                                  */
 
+#include "internal.h"
+
 #include <glib.h>
 #if GLIB_CHECK_VERSION(2,6,0)
 #	include <glib/gstdio.h>
@@ -134,9 +136,9 @@ void PE_debug_dump_keyring(key_ring * ring) {
 
    for( i = ring; i != NULL; i = i->next ) {
       purple_debug(PURPLE_DEBUG_INFO, "pidgin-encryption", "Key ring::%*s::%p\n",
-                 sizeof(((key_ring_data *)i->data)->name),
-                 ((key_ring_data *)i->data)->name,
-                 ((key_ring_data *)i->data)->account);
+                   (unsigned)sizeof(((key_ring_data *)i->data)->name),
+                   ((key_ring_data *)i->data)->name,
+                   ((key_ring_data *)i->data)->account);
    }
 }
 
@@ -295,8 +297,22 @@ static const char * get_base_key_path() {
    if (strcmp(basepath, purple_user_dir()) == 0) {
       purple_prefs_set_string("/plugins/gtk/encrypt/key_path", "");      
       basepath = 0;
+   } else {
+      // see if the basepath matches purple_user_dir with .purple -> .gaim.  If so, migrate it
+      gchar ** splitPath = g_strsplit(purple_user_dir(), ".purple", 5);
+      gchar * legacyPath = g_strjoinv(".gaim", splitPath);
+
+      if (strcmp(basepath, legacyPath) == 0) {
+         purple_prefs_set_string("/plugins/gtk/encrypt/key_path", "");      
+         basepath = 0;
+      }
+
+      g_strfreev(splitPath);
+      g_free(legacyPath);
    }
 
+   
+
    if (!basepath || *basepath == 0) {
       basepath = purple_user_dir();
       if (!displayedpath || strcmp(basepath, displayedpath) != 0) {
@@ -354,8 +370,16 @@ static KeyCheckVal PE_check_known_key(const char* filename, key_ring_data* key)
 #ifdef S_IWGRP
       /* WIN32 doesn't have user-based file permissions, so skips this */
       if (fs.st_mode & (S_IWGRP | S_IWOTH)) {
-         purple_debug(PURPLE_DEBUG_ERROR, "pidgin-encryption", "Invalid permissions, rejecting file: %s\n", path);
-         return KEY_CONFLICT;
+         fd = g_open(path, O_WRONLY, 0);
+         if (fd == -1) {
+            /* Ok, maybe something else strange is going on... */
+            purple_debug(PURPLE_DEBUG_ERROR, "pidgin-encryption", "Error trying to modify permissions on known key file\n");
+            return KEY_NOT_THERE;
+         }
+         fstat(fd, &fs);
+         fchmod(fd, fs.st_mode & S_IRWXU);  /* zero out non-owner permissions */
+         close(fd);
+         purple_debug(PURPLE_DEBUG_WARNING, "pidgin-encryption", "Changed file permissions on %s\n", path);
       }
 #endif
    }
@@ -446,20 +470,21 @@ void PE_add_key_to_file(const char *filename, key_ring_data* key) {
          PE_ui_error(errbuf);
          return;
       } else {
-         purple_debug(PURPLE_DEBUG_ERROR, "pidgin-encryption", "Key file '%s' no longer read-only.\n");
+         purple_debug(PURPLE_DEBUG_ERROR, "pidgin-encryption", "Key file '%s' no longer read-only.\n", path);
       }
    }
    fstat(fd, &fdstat);
 #ifdef S_IRWXG
    /* WIN32 doesn't have user-based file permissions, so skips this */
    if (fdstat.st_mode & (S_IRWXG | S_IRWXO)) {
-      purple_debug(PURPLE_DEBUG_ERROR, "pidgin-encryption", "Bad permissions on key file: %s\n", path);
-      purple_debug(PURPLE_DEBUG_ERROR, "pidgin-encryption", "I won't save to a world-accesible key file.\n");
+      fchmod(fd, fdstat.st_mode & S_IRWXU);  /* zero out non-owner permissions */
+
+      purple_debug(PURPLE_DEBUG_ERROR, "pidgin-encryption", "Bad permissions on key file.  Changed: %s\n", path);
       g_snprintf(errbuf, sizeof(errbuf),
-                 _("Bad permissions on key file: %s\nPidgin-Encryption will not save keys to a world- or group-accessible file."),
+                 _("Permissions on key file changed for: %s\n"
+                   "Pidgin-Encryption will not save keys to a world- or group-accessible file."),
                    filename);
       PE_ui_error(errbuf);
-      return;
    } 
 #endif
 
@@ -703,7 +728,32 @@ key_ring * PE_load_keys(const char *filename) {
    PurpleAccount* account;
 
    g_snprintf(path, sizeof(path), "%s%s%s", get_base_key_path(), G_DIR_SEPARATOR_S, filename);
-   if( (fp = g_fopen(path, "r")) != NULL ) {
+
+   /* Check permissions on file before use */
+#ifdef S_IRWXG
+   /* WIN32 doesn't have user-based file permissions, so skips this */
+   {
+      char errbuf[500];
+      struct stat fdstat;
+      int fd = g_open(path, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
+      if (fd != -1) {
+         fstat(fd, &fdstat);
+         if (fdstat.st_mode & (S_IRWXG | S_IRWXO)) {
+            fchmod(fd, fdstat.st_mode & S_IRWXU);  /* zero out non-owner permissions */
+            
+            purple_debug(PURPLE_DEBUG_ERROR, "pidgin-encryption", "Bad permissions on key file.  Changed: %s\n", path);
+            g_snprintf(errbuf, sizeof(errbuf),
+                       _("Permissions on key file changed for: %s\n"
+                         "Pidgin-Encryption will not use keys in a world- or group-accessible file."),
+                       filename);
+            PE_ui_error(errbuf);
+         } 
+         close(fd);
+      }
+   }
+#endif
+
+if( (fp = g_fopen(path, "r")) != NULL ) {
       do {
       	 /* 7999 = MAX_KEY_STORLEN - 1 */
          rv = fscanf(fp, "%163s %9s %9s %7999s\n", nameacct, proto_name,
@@ -754,7 +804,7 @@ key_ring * PE_load_keys(const char *filename) {
             new_key->account = account;
             strncpy(new_key->name, name, sizeof(new_key->name));
             purple_debug(PURPLE_DEBUG_MISC, "pidgin-encryption", "load_keys() %i: Added: %*s %s %s\n", __LINE__,
-                         sizeof(new_key->name), new_key->name, proto_name, proto_ver);
+                         (unsigned)sizeof(new_key->name), new_key->name, proto_name, proto_ver);
 
             new_ring = g_slist_append(new_ring, new_key);
          } else if (rv > 0) {
